""" Config for the overall application """
from __future__ import absolute_import, print_function, unicode_literals

from django.apps.config import AppConfig


class DjCoreConfig(AppConfig):
    name = label = 'dj_core'
